import base64


def processRequest(req):
    raise NotImplementedError("processRequest not implemented")


def handler(event, context):
    print("triggered by HttpHandler...")
    print("event is %s" % event)

    if event["httpMethod"] == "OPTIONS":
        print(
            "NOTICE!, received options preflight request, may consider use api-gateway mock options method instead...")
        return {
            "statusCode": 200,
            "headers": {
                "Access-Control-Allow-Origin": "*",
                "Access-Control-Allow-Methods": "POST, GET, OPTIONS",
                "Access-Control-Allow-Headers": "Content-Type,X-Amz-Date,X-Api-Key,X-Amz-Security-Token",
                "Content-Type": "application/json"
            },
            "body": None,
            "isBase64Encoded": False
        }

    try:
        if event["body"]:
            requestBody = base64.b64encode(bytearray.fromhex(event["body"])) if event["isBase64Encoded"] else event["body"]

        req = {
            "headers": event["headers"] or {},
            "queryStringParameters": event["queryStringParameters"] or {},
            "pathParameters": event["pathParameters"] or {},
            "body": requestBody if "requestBody" in locals() else {}
        }

        resp = processRequest(req)
        if not isinstance(resp, dict):
            raise Exception("resp type must be dict, now is %s" % type(resp))

        if resp and "contentType" in resp:
            return {
                "statusCode": 200,
                "headers": {
                    "Access-Control-Allow-Origin": "*",
                    "Access-Control-Allow-Methods": "POST, GET, OPTIONS",
                    "Access-Control-Allow-Headers": "Content-Type,X-Amz-Date,X-Api-Key,X-Amz-Security-Token",
                    "Content-Type": resp["contentType"]
                },
                "body": resp["body"],
                "isBase64Encoded": resp["isBase64Encoded"]
            }
        else:
            return {
                "statusCode": 200,
                "headers": {
                    "Access-Control-Allow-Origin": "*",
                    "Access-Control-Allow-Methods": "POST, GET, OPTIONS",
                    "Access-Control-Allow-Headers": "Content-Type,X-Amz-Date,X-Api-Key,X-Amz-Security-Token",
                    "Content-Type": "application/json"
                },
                "body": resp,
                "isBase64Encoded": False
            }
    except Exception as ex:
        print(ex)
        return {
            "headers": {
                "Access-Control-Allow-Origin": "*",
                "Access-Control-Allow-Methods": "POST, GET, OPTIONS",
                "Access-Control-Allow-Headers": "Content-Type,X-Amz-Date,X-Api-Key,X-Amz-Security-Token",
                "Content-Type": "application/json"
            },
            "statusCode": 200,
            "body": ex,
            "isBase64Encoded": False
        }


if __name__ == "__main__":
    handler(None, None)



import datetime
import boto3
import traceback
from copy import deepcopy

from util.ptLogger import PTLogger
from util.ptMetric import PTMetric

client = boto3.client('events')
logger = PTLogger(__name__).get_logger()
global_parameters = {
    "EnableNotification": False,
    "Annotation": {
        "Application": "undefined",
        "Module": "undefined",
        "Creator": "undefined",
        "Owner": "undefined",
        "Reviser": "undefined"
    }
}


def processRequest(messageList, event, context):
    raise NotImplementedError("processRequest not implemented")


def _extract_message(record):
    eventSourceArn = record["eventSourceARN"]
    queueName = eventSourceArn.split(":")[-1]
    messageBodyText = record.get("body")
    message = {
        "receiptHandle": record.get("receiptHandle"),
        "messageBodyText": messageBodyText,
        "metadata": {
            "eventSource": record.get("eventSource"),
            "eventSourceARN": record.get("eventSourceARN"),
            "queueName": queueName,
            "senderId": record.get("attributes").get("SenderId"),
            "approximateFirstReceiveTimestamp": record.get("attributes").get("ApproximateFirstReceiveTimestamp"),
            "approximateReceiveCount": record.get("attributes").get("ApproximateReceiveCount"),
            "sentTimestamp": record.get("attributes").get("SentTimestamp"),
        }
    }
    return message


def handler(event, context):
    tick_start = datetime.datetime.now()
    logger.info("triggered by QueueHandler...")
    logger.info(f"Lambda function_name: {context.function_name}")
    logger.info(f"Lambda function_version: {context.function_version}")
    logger.info(f"Lambda invoked_function_arn: {context.invoked_function_arn}")
    logger.info(f"message is: {event}")
    logger.info("initializing ptMetric...")

    dimensions = deepcopy(global_parameters["Annotation"])
    dimensions.update({"FunctionName": context.function_name})
    dimensions.update({"TriggerHandlerType": "QueueTrigger"})

    metric_wrapper = PTMetric(namespace="PT/Lambda", dimensions=dimensions)
    metric_wrapper.put_metric_data(name="Entry", value=1, unit="Count")

    try:
        messageList = [_extract_message(r) for r in event["Records"]]

        respDict = processRequest(messageList, event, context)
        if respDict and not isinstance(respDict, dict):
            raise Exception("resp type must be dict, now is %s" % type(respDict))

        metric_wrapper.put_metric_data(name="Invocation", value=1, unit="Count")
        tick_end = datetime.datetime.now()
        tick_delta = (tick_end - tick_start).total_seconds() * 1000
        metric_wrapper.put_metric_data(name="Duration", value=tick_delta, unit="Milliseconds")

        if global_parameters["EnableNotification"]:
            logger.warn("notification is not implemented")

    except Exception as ex:
        metric_wrapper.put_metric_data(name="Error", value=1, unit="Count")
        traceback.print_exc()
        if global_parameters["EnableNotification"]:
            logger.warn("notification is not implemented")



import datetime
import boto3
import traceback
import json

from copy import deepcopy

from util.ptLogger import PTLogger
from util.ptMetric import PTMetric
from util.ptTrace import PTTrace

client = boto3.client('events')
logger = PTLogger(__name__).get_logger()
global_parameters = {
    "EnableNotification": False,
    "EnableMetric": True,
    "EnableTrace": True,  # JimmyMo: must add pt-infra-xray layer
    "Annotation": {
        "Application": "undefined",
        "Module": "undefined",
        "Creator": "undefined",
        "Owner": "undefined",
        "Reviser": "undefined"
    }
}


def processRequest(req, event=None, context=None):
    raise NotImplementedError("processRequest not implemented")


def handler(event, context):
    tick_start = datetime.datetime.now()
    logger.info("triggered by EventHandler...")
    logger.info(f"Lambda function_name: {context.function_name}")
    logger.info(f"Lambda function_version: {context.function_version}")
    logger.info(f"Lambda invoked_function_arn: {context.invoked_function_arn}")
    logger.info(f"event is: {event}")

    if global_parameters["EnableMetric"]:
        logger.info("metric enabled! initializing ptMetric...")
        dimensions = deepcopy(global_parameters["Annotation"])
        dimensions.update({"FunctionName": context.function_name})
        dimensions.update({"TriggerHandlerType": "EventTrigger"})

        context.metric_wrapper = PTMetric(namespace="PT/Lambda", dimensions=dimensions)
        context.metric_wrapper.put_metric_data(name="Entry", value=1, unit="Count")

    if global_parameters["EnableTrace"]:
        logger.info("trace enabled!")
        context.trace_wrapper = PTTrace()
        context.trace_wrapper.enable_all_trace()

    if isinstance(event["detail"], str):
        eventDetail = json.loads(event["detail"])
    else:
        eventDetail = event["detail"]

    try:
        req = {
            "source": event["source"],
            "detailType": event["detail-type"],
            "metadata": eventDetail.get("metadata", {}),
            "payload": eventDetail.get("payload", {}),
        }
        with context.trace_wrapper.record("ProcessRequest") as recorder:
            recorder.put_metadata("very_important_inforamtion", {"message": "Hello I am metadata"}, "information")
            recorder.put_annotation("very_important_annotation", "Very very important information")
            respDict = processRequest(req, event, context)
            if respDict and not isinstance(respDict, dict):
                raise Exception("resp type must be dict, now is %s" % type(respDict))

        if global_parameters["EnableMetric"]:
            context.metric_wrapper.put_metric_data(name="Invocation", value=1, unit="Count")
            tick_end = datetime.datetime.now()
            tick_delta = (tick_end - tick_start).total_seconds() * 1000
            context.metric_wrapper.put_metric_data(name="Duration", value=tick_delta, unit="Milliseconds")

        if global_parameters["EnableNotification"]:
            logger.warn("notification is not implemented")
    except Exception as ex:
        context.metric_wrapper.put_metric_data(name="Error", value=1, unit="Count") if global_parameters["EnableMetric"] else None
        traceback.print_exc()
        if global_parameters["EnableNotification"]:
            logger.warn("notification is not implemented")
        raise ex


if __name__ == "__main__":
    print("Done")


from setuptools import setup, find_packages

setup(name='pt_lambda_wrapper',
      version='0.1.8',
      description='trigger wrapper for aws lambda application',
      author='JimmyMo',
      author_email='jocund_mo@aliyun.com',
      install_requires=['pt_lambda_util'],
      package_dir={'pt_lambda_wrapper': 'src/pt_lambda_wrapper'},
      packages=["pt_lambda_wrapper"],
      license="Apache License 2.0"
      )
